#ifndef COAP_H
#define COAP_H

#define COAP_PORT 61616

#define COAP_VERSION 1

#define TYPE_CON 0
#define TYPE_NON 1
#define TYPE_ACK 2
#define TYPE_RST 3

#define METHOD_GET 1
#define METHOD_POST 2
#define METHOD_PUT 3
#define METHOD_DELETE 4


#define ERROR_CODE(major, minor)  ((major & 0x7) << 5 | (minor & 0x1f))

#define ERROR_100      ERROR_CODE(1,  0)   /* 100 Continue */
#define ERROR_200      ERROR_CODE(2,  0)   /* 200 OK */
#define ERROR_201      ERROR_CODE(2,  1)   /* 201 Created */
#define ERROR_205      ERROR_CODE(2,  5)   /* 201 Created */
#define ERROR_304      ERROR_CODE(3,  4)   /* 304 Not Modified */
#define ERROR_400      ERROR_CODE(4,  0)   /* 400 Bad Request */
#define ERROR_404      ERROR_CODE(4,  4)   /* 404 Not Found */
#define ERROR_405      ERROR_CODE(4,  5)   /* 405 Method Not Allowed */
#define ERROR_415      ERROR_CODE(4, 15)   /* 415 Unsupported Media Type */
#define ERROR_500      ERROR_CODE(5,  0)   /* 500 Internal Server Error */
#define ERROR_503      ERROR_CODE(5,  3)   /* 503 Service Unavailable */
#define ERROR_504      ERROR_CODE(5,  4)   /* 504 Gateway Timeout */

#define CT_PLAIN_FORMAT 0
#define CT_LINK_FORMAT 40

#define OPT_CONTENT_TYPE 1
#define OPT_MAX_AGE 2
#define OPT_URI_SCHEME 3
#define OPT_ETAG 4
#define OPT_URI_AUTHORITY 5
#define OPT_LOCATION 6
#define OPT_URI_PATH 9
#define OPT_SUB_LIFETIME 10
#define OPT_MAX 11

#define COAP_WAIT 10 /* secondes */

typedef char *(*reply_function)(unsigned char, char *, unsigned short);

typedef struct {
  char *url;
  reply_function reply;
} observer_funcs;

/*typedef struct {
  uip_ipaddr_t addr;
  unsigned short port;
  short timeout;
  const observer_funcs *service;
  unsigned short old_value;
} observer;*/

void send_reply(uip_ipaddr_t *addr, unsigned short port, unsigned short size);

void coap_init();
void coap_handler();
char *coap_get_not_found(unsigned char, char *, unsigned short);
char *coap_make_reset();
char *coap_make_announce();
char *coap_make_base_reply(unsigned char type);
char *coap_make_error_reply(unsigned char error);
char *coap_make_post_request(unsigned char type, char *url);
const observer_funcs *get_from_url(char *url, unsigned short size);

#define UIP_IP_BUF   ((struct uip_ip_hdr *)&uip_buf[UIP_LLH_LEN])
#define UIP_UDP_BUF   ((struct uip_udp_hdr *)&uip_buf[UIP_LLIPH_LEN])

#endif
